#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <limits.h>

#define MAX_ARRAY_SIZE 1000
#define MAX_STRING_LENGTH 256
#define PI 3.14159
#define MIN_MATRIX_SIZE 1
#define MAX_MATRIX_SIZE 100
#define MAX_RECURSION_DEPTH 1000

int globalCounter = 0;
char* globalErrorMessage = NULL;
int recursionDepth = 0;

const char* getErrorMessage(void);
void setErrorMessage(const char* message);

typedef enum {
    SUCCESS = 0,
    ERROR_INVALID_INPUT,
    ERROR_MEMORY_ALLOCATION,
    ERROR_ARRAY_BOUNDS,
    ERROR_DIVISION_BY_ZERO,
    ERROR_OVERFLOW,
    ERROR_RECURSION_LIMIT,
    ERROR_NULL_POINTER,
    ERROR_INVALID_OPERATION
} ErrorCode;

struct Point {
    int x;
    int y;
    char label[MAX_STRING_LENGTH];
    double weight;
};

struct Circle {
    struct Point center;
    double radius;
    int valid;
    char type[MAX_STRING_LENGTH];
};

struct Matrix {
    int** data;
    int rows;
    int cols;
    int isSquare;
    double determinant;
};

typedef struct {
    int* array;
    int size;
    int capacity;
    int isSorted;
    int lastOperation;
} DynamicArray;

typedef struct Node {
    int data;
    struct Node* next;
    struct Node* prev;
    int visited;
    int depth;
} Node;

const char* getErrorMessage(void) {
    return globalErrorMessage ? globalErrorMessage : "No error";
}

void setErrorMessage(const char* message) {
    if (globalErrorMessage != NULL) {
        free(globalErrorMessage);
    }
    if (message != NULL) {
        globalErrorMessage = strdup(message);
        if (globalErrorMessage == NULL) {
            fprintf(stderr, "Critical: Failed to allocate memory for error message\n");
        }
    } else {
        globalErrorMessage = NULL;
    }
}

int factorial(int n) {
    if (++recursionDepth > MAX_RECURSION_DEPTH) {
        setErrorMessage("Maximum recursion depth exceeded");
        recursionDepth--;
        return -1;
    }

    if (n < 0) {
        setErrorMessage("Factorial of negative number");
        recursionDepth--;
        return -1;
    }

    // Check for overflow using a rough approximation
    if (n > 20) {
        setErrorMessage("Input too large, potential overflow");
        recursionDepth--;
        return -1;
    }

    int result;
    if (n <= 1) {
        result = 1;
    } else {
        int temp = factorial(n - 1);
        if (temp == -1) {
            recursionDepth--;
            return -1;
        }
        
        // Check for multiplication overflow
        if (temp > INT_MAX / n) {
            setErrorMessage("Factorial calculation overflow");
            recursionDepth--;
            return -1;
        }
        
        result = n * temp;
    }

    recursionDepth--;
    return result;
}

DynamicArray* createDynamicArray(int initialCapacity) {
    if (initialCapacity <= 0) {
        setErrorMessage("Invalid initial capacity");
        return NULL;
    }

    if (initialCapacity > MAX_ARRAY_SIZE) {
        setErrorMessage("Initial capacity exceeds maximum allowed size");
        return NULL;
    }

    DynamicArray* arr = (DynamicArray*)malloc(sizeof(DynamicArray));
    if (!arr) {
        setErrorMessage("Memory allocation failed for DynamicArray struct");
        return NULL;
    }

    arr->array = (int*)malloc(sizeof(int) * initialCapacity);
    if (!arr->array) {
        free(arr);
        setErrorMessage("Memory allocation failed for array");
        return NULL;
    }

    arr->size = 0;
    arr->capacity = initialCapacity;
    arr->isSorted = 1;  // Empty array is considered sorted
    arr->lastOperation = 0;

    return arr;
}

void pushBack(DynamicArray* arr, int value) {
    if (!arr) {
        setErrorMessage("Null array pointer");
        return;
    }

    if (arr->size >= MAX_ARRAY_SIZE) {
        setErrorMessage("Array size limit reached");
        return;
    }

    if (arr->size >= arr->capacity) {
        int newCapacity = arr->capacity * 2;
        if (newCapacity > MAX_ARRAY_SIZE) {
            newCapacity = MAX_ARRAY_SIZE;
        }

        int* newArray = (int*)realloc(arr->array, sizeof(int) * newCapacity);
        if (!newArray) {
            setErrorMessage("Memory reallocation failed");
            return;
        }
        arr->array = newArray;
        arr->capacity = newCapacity;
    }

    // Check if this insert will break the sorted property
    if (arr->isSorted && arr->size > 0 && value < arr->array[arr->size - 1]) {
        arr->isSorted = 0;
    }

    arr->array[arr->size++] = value;
    arr->lastOperation = 1; // Insert operation
}

void quickSort(int arr[], int left, int right, int* error) {
    if (++recursionDepth > MAX_RECURSION_DEPTH) {
        setErrorMessage("Maximum recursion depth exceeded in quickSort");
        *error = 1;
        recursionDepth--;
        return;
    }

    if (!arr) {
        setErrorMessage("Null array pointer in quickSort");
        *error = 1;
        recursionDepth--;
        return;
    }

    if (left < 0 || right < 0 || left >= MAX_ARRAY_SIZE || right >= MAX_ARRAY_SIZE) {
        setErrorMessage("Array index out of bounds in quickSort");
        *error = 1;
        recursionDepth--;
        return;
    }

    if (left < right) {
        // Choose pivot as median of three
        int mid = left + (right - left) / 2;
        int pivot_idx;
        
        if (arr[left] <= arr[mid]) {
            if (arr[mid] <= arr[right]) {
                pivot_idx = mid;
            } else if (arr[left] <= arr[right]) {
                pivot_idx = right;
            } else {
                pivot_idx = left;
            }
        } else {
            if (arr[left] <= arr[right]) {
                pivot_idx = left;
            } else if (arr[mid] <= arr[right]) {
                pivot_idx = right;
            } else {
                pivot_idx = mid;
            }
        }

        // Swap pivot to end
        int temp = arr[right];
        arr[right] = arr[pivot_idx];
        arr[pivot_idx] = temp;

        int pivot = arr[right];
        int i = left - 1;
        
        for (int j = left; j < right; j++) {
            if (arr[j] <= pivot) {
                i++;
                temp = arr[i];
                arr[i] = arr[j];
                arr[j] = temp;
                globalCounter++;
            }
        }

        temp = arr[i + 1];
        arr[i + 1] = arr[right];
        arr[right] = temp;

        // Check for errors in recursive calls
        quickSort(arr, left, i, error);
        if (*error) {
            recursionDepth--;
            return;
        }

        quickSort(arr, i + 2, right, error);
        if (*error) {
            recursionDepth--;
            return;
        }
    }

    recursionDepth--;
}

struct Matrix* createMatrix(int rows, int cols) {
    if (rows < MIN_MATRIX_SIZE || cols < MIN_MATRIX_SIZE) {
        setErrorMessage("Matrix dimensions too small");
        return NULL;
    }

    if (rows > MAX_MATRIX_SIZE || cols > MAX_MATRIX_SIZE) {
        setErrorMessage("Matrix dimensions exceed maximum allowed size");
        return NULL;
    }

    struct Matrix* matrix = (struct Matrix*)malloc(sizeof(struct Matrix));
    if (!matrix) {
        setErrorMessage("Memory allocation failed for matrix struct");
        return NULL;
    }

    matrix->rows = rows;
    matrix->cols = cols;
    matrix->isSquare = (rows == cols);
    matrix->determinant = 0.0;
    
    matrix->data = (int**)malloc(rows * sizeof(int*));
    if (!matrix->data) {
        free(matrix);
        setErrorMessage("Memory allocation failed for matrix rows");
        return NULL;
    }

    for (int i = 0; i < rows; i++) {
        matrix->data[i] = (int*)calloc(cols, sizeof(int));
        if (!matrix->data[i]) {
            for (int j = 0; j < i; j++) {
                free(matrix->data[j]);
            }
            free(matrix->data);
            free(matrix);
            setErrorMessage("Memory allocation failed for matrix columns");
            return NULL;
        }
    }

    return matrix;
}

struct Matrix* multiplyMatrices(struct Matrix* m1, struct Matrix* m2) {
    if (!m1 || !m2) {
        setErrorMessage("Null matrix pointer");
        return NULL;
    }

    if (!m1->data || !m2->data) {
        setErrorMessage("Invalid matrix data");
        return NULL;
    }

    if (m1->cols != m2->rows) {
        setErrorMessage("Invalid matrix dimensions for multiplication");
        return NULL;
    }

    if (m1->cols > MAX_MATRIX_SIZE || m2->rows > MAX_MATRIX_SIZE) {
        setErrorMessage("Matrix dimensions exceed maximum allowed size");
        return NULL;
    }

    struct Matrix* result = createMatrix(m1->rows, m2->cols);
    if (!result) return NULL;

    // Check for potential integer overflow in multiplication
    long maxPossibleValue = 0;
    for (int i = 0; i < m1->rows; i++) {
        for (int j = 0; j < m2->cols; j++) {
            long sum = 0;
            for (int k = 0; k < m1->cols; k++) {
                long prod = (long)m1->data[i][k] * m2->data[k][j];
                sum += prod;
                if (sum > INT_MAX || sum < INT_MIN) {
                    setErrorMessage("Integer overflow in matrix multiplication");
                    // Clean up
                    for (int x = 0; x < result->rows; x++) {
                        free(result->data[x]);
                    }
                    free(result->data);
                    free(result);
                    return NULL;
                }
            }
            result->data[i][j] = (int)sum;
            if (abs(result->data[i][j]) > maxPossibleValue) {
                maxPossibleValue = abs(result->data[i][j]);
            }
        }
    }

    result->isSquare = (result->rows == result->cols);
    return result;
}

Node* insertNode(Node* head, int data) {
    static int maxDepth = 0;

    Node* newNode = (Node*)malloc(sizeof(Node));
    if (!newNode) {
        setErrorMessage("Memory allocation failed for new node");
        return head;
    }

    newNode->data = data;
    newNode->next = NULL;
    newNode->prev = NULL;
    newNode->visited = 0;
    
    if (!head) {
        newNode->depth = 0;
        maxDepth = 0;
        return newNode;
    }

    // Find the appropriate position to insert (keeping list sorted)
    Node* current = head;
    Node* prev = NULL;
    int currentDepth = 0;

    while (current && current->data < data) {
        prev = current;
        current = current->next;
        currentDepth++;

        if (currentDepth > MAX_ARRAY_SIZE) {
            setErrorMessage("List exceeds maximum allowed size");
            free(newNode);
            return head;
        }
    }

    // Update maxDepth if necessary
    if (currentDepth > maxDepth) {
        maxDepth = currentDepth;
    }

    // Insert the new node
    newNode->depth = currentDepth;
    newNode->next = current;
    
    if (prev) {
        prev->next = newNode;
        newNode->prev = prev;
    } else {
        return newNode; // New head
    }

    if (current) {
        current->prev = newNode;
    }

    return head;
}

static int calculationResult = 0;

void performSimpleCalculations() {
    int tempValue = 100;  // 初始值
    
    // 第一个条件：检查是否大于50
    if (tempValue > 50) {
        calculationResult += tempValue * 2;
    } else {
        calculationResult += tempValue / 2;
    }
    
    // 第二个条件：检查是否能被3整除
    if (tempValue % 3 == 0) {
        calculationResult *= 3;
    } else {
        calculationResult += 3;
    }
    
    // 第三个条件：检查是否在特定范围内
    if (calculationResult >= 150 && calculationResult <= 300) {
        calculationResult -= 50;
    } else {
        calculationResult += 50;
    }
    
    // 第四个条件：检查是否为偶数
    if (calculationResult % 2 == 0) {
        calculationResult /= 2;
    } else {
        calculationResult *= 2;
    }
    
    // 第五个条件：检查个位数
    if (calculationResult % 10 < 5) {
        calculationResult += 5;
    } else {
        calculationResult -= 5;
    }
    
    // 第六个条件：最终范围检查
    if (calculationResult > 1000) {
        calculationResult = 1000;
    } else {
        calculationResult += 10;
    }
    
    // 打印最终结果
    printf("Final calculation result: %d\n", calculationResult);
}

int main() {
    srand(time(NULL));
    
    // Initialize dynamic array with error checking
    DynamicArray* dynArr = createDynamicArray(10);
    if (!dynArr) {
        printf("Error: %s\n", getErrorMessage());
        return -1;
    }
    
    // Test array operations with boundary checks
    for (int i = 0; i < 15; i++) {
        pushBack(dynArr, rand() % 100);
        if (globalErrorMessage) {
            printf("Error during array push: %s\n", getErrorMessage());
            break;
        }
    }
    
    // Matrix operations with error handling
    struct Matrix* matrix1 = createMatrix(3, 3);
    struct Matrix* matrix2 = createMatrix(3, 3);
    if (!matrix1 || !matrix2) {
        printf("Error: %s\n", getErrorMessage());
        // Cleanup
        if (matrix1) {
            free(matrix1);
        }
        if (matrix2) {
            free(matrix2);
        }
        free(dynArr->array);
        free(dynArr);
        return -1;
    }
    
    // Initialize matrices with random data
    for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            matrix1->data[i][j] = rand() % 10;
            matrix2->data[i][j] = rand() % 10;
        }
    }
    
    // Matrix multiplication with error checking
    struct Matrix* result = multiplyMatrices(matrix1, matrix2);
    if (!result) {
        printf("Error: %s\n", getErrorMessage());
        // Cleanup
        for (int i = 0; i < matrix1->rows; i++) {
            free(matrix1->data[i]);
            free(matrix2->data[i]);
        }
        free(matrix1->data);
        free(matrix2->data);
        free(matrix1);
        free(matrix2);
        free(dynArr->array);
        free(dynArr);
        return -1;
    }
    
    // Linked list operations with error handling
    Node* head = NULL;
    for (int i = 0; i < 5; i++) {
        head = insertNode(head, rand() % 50);
        if (globalErrorMessage) {
            printf("Error during list insertion: %s\n", getErrorMessage());
            break;
        }
    }
    
    // Memory cleanup
    free(dynArr->array);
    free(dynArr);
    
    for (int i = 0; i < matrix1->rows; i++) {
        free(matrix1->data[i]);
        free(matrix2->data[i]);
        free(result->data[i]);
    }
    free(matrix1->data);
    free(matrix2->data);
    free(result->data);
    free(matrix1);
    free(matrix2);
    free(result);
    
    // 增强的链表清理，包含循环检测
    if (head) {
        Node* slow = head;
        Node* fast = head;
        int hasLoop = 0;
        
        // Floyd's cycle detection
        while (fast && fast->next) {
            slow = slow->next;
            fast = fast->next->next;
            if (slow == fast) {
                hasLoop = 1;
                break;
            }
        }
        
        if (hasLoop) {
            setErrorMessage("Circular reference detected in linked list");
            // 特殊的循环链表清理
            Node* current = head;
            int maxNodes = 1000; // 防止无限循环
            int nodeCount = 0;
            
            while (current && nodeCount < maxNodes) {
                if (current->visited) {
                    break;
                }
                current->visited = 1;
                current = current->next;
                nodeCount++;
            }
        }
        
        // 标准清理过程
        while (head) {
            Node* temp = head;
            head = head->next;
            free(temp);
        }
    }
    
    if (globalErrorMessage) {
        free(globalErrorMessage);
    }
    performSimpleCalculations();
    
    return 0;
}

// 增强的几何计算函数
double calculateDistance(struct Point p1, struct Point p2, ErrorCode* error) {
    *error = SUCCESS;
    
    // 验证权重
    if (p1.weight <= 0 || p2.weight <= 0) {
        *error = ERROR_INVALID_INPUT;
        setErrorMessage("Invalid point weights");
        return -1.0;
    }
    
    // 检查坐标范围
    if (abs(p1.x) > MAX_ARRAY_SIZE || abs(p1.y) > MAX_ARRAY_SIZE || 
        abs(p2.x) > MAX_ARRAY_SIZE || abs(p2.y) > MAX_ARRAY_SIZE) {
        *error = ERROR_ARRAY_BOUNDS;
        setErrorMessage("Coordinates out of valid range");
        return -1.0;
    }
    
    // 计算带权重的距离
    double dx = (p2.x - p1.x) * sqrt(p2.weight / p1.weight);
    double dy = (p2.y - p1.y) * sqrt(p2.weight / p1.weight);
    
    // 检查溢出
    if (fabs(dx) > MAX_ARRAY_SIZE || fabs(dy) > MAX_ARRAY_SIZE) {
        *error = ERROR_OVERFLOW;
        setErrorMessage("Distance calculation overflow");
        return -1.0;
    }
    
    double distance = sqrt(dx*dx + dy*dy);
    
    // 验证结果
    if (isnan(distance) || isinf(distance)) {
        *error = ERROR_INVALID_OPERATION;
        setErrorMessage("Invalid distance calculation result");
        return -1.0;
    }
    
    return distance;
}

double calculateCircleArea(struct Circle circle, ErrorCode* error) {
    *error = SUCCESS;
    
    // 验证圆的有效性
    if (!circle.valid) {
        *error = ERROR_INVALID_INPUT;
        setErrorMessage("Invalid circle");
        return -1.0;
    }
    
    // 验证半径
    if (circle.radius <= 0) {
        *error = ERROR_INVALID_INPUT;
        setErrorMessage("Invalid circle radius");
        return -1.0;
    }
    
    // 检查半径是否过大
    if (circle.radius > MAX_ARRAY_SIZE) {
        *error = ERROR_ARRAY_BOUNDS;
        setErrorMessage("Circle radius too large");
        return -1.0;
    }
    
    // 验证圆心坐标
    if (abs(circle.center.x) > MAX_ARRAY_SIZE || abs(circle.center.y) > MAX_ARRAY_SIZE) {
        *error = ERROR_ARRAY_BOUNDS;
        setErrorMessage("Circle center coordinates out of valid range");
        return -1.0;
    }
    
    // 计算面积
    double area = PI * circle.radius * circle.radius;
    
    // 检查结果
    if (isnan(area) || isinf(area)) {
        *error = ERROR_OVERFLOW;
        setErrorMessage("Area calculation overflow");
        return -1.0;
    }
    
    // 验证圆的类型
    if (strcmp(circle.type, "standard") == 0) {
        return area;
    } else if (strcmp(circle.type, "weighted") == 0) {
        // 加权面积计算
        if (circle.center.weight <= 0) {
            *error = ERROR_INVALID_INPUT;
            setErrorMessage("Invalid weight for weighted circle");
            return -1.0;
        }
        return area * circle.center.weight;
    } else {
        *error = ERROR_INVALID_INPUT;
        setErrorMessage("Unknown circle type");
        return -1.0;
    }
}

// 新增的矩阵验证函数
ErrorCode validateMatrix(struct Matrix* matrix) {
    if (!matrix) {
        setErrorMessage("Null matrix pointer");
        return ERROR_NULL_POINTER;
    }
    
    if (!matrix->data) {
        setErrorMessage("Null matrix data");
        return ERROR_NULL_POINTER;
    }
    
    if (matrix->rows < MIN_MATRIX_SIZE || matrix->cols < MIN_MATRIX_SIZE) {
        setErrorMessage("Matrix dimensions too small");
        return ERROR_INVALID_INPUT;
    }
    
    if (matrix->rows > MAX_MATRIX_SIZE || matrix->cols > MAX_MATRIX_SIZE) {
        setErrorMessage("Matrix dimensions too large");
        return ERROR_ARRAY_BOUNDS;
    }
    
    // 验证所有行的内存分配
    for (int i = 0; i < matrix->rows; i++) {
        if (!matrix->data[i]) {
            setErrorMessage("Invalid matrix row pointer");
            return ERROR_NULL_POINTER;
        }
    }
    
    // 验证矩阵是否为方阵的标志是否正确
    if ((matrix->rows == matrix->cols) != matrix->isSquare) {
        setErrorMessage("Inconsistent square matrix flag");
        return ERROR_INVALID_INPUT;
    }
    
    return SUCCESS;
}

// 新增的数组操作函数
void removeDuplicates(DynamicArray* arr) {
    if (!arr || !arr->array) {
        setErrorMessage("Invalid array");
        return;
    }
    
    if (arr->size <= 1) {
        return;
    }
    
    // 先排序以优化去重过程
    int error = 0;
    quickSort(arr->array, 0, arr->size - 1, &error);
    if (error) {
        return;
    }
    
    int writeIndex = 1;
    for (int readIndex = 1; readIndex < arr->size; readIndex++) {
        if (arr->array[readIndex] != arr->array[readIndex - 1]) {
            arr->array[writeIndex] = arr->array[readIndex];
            writeIndex++;
        }
    }
    
    arr->size = writeIndex;
    arr->isSorted = 1;
}

// 新增的点集操作函数
struct Point* findCentroid(struct Point* points, int count, ErrorCode* error) {
    *error = SUCCESS;
    
    if (!points || count <= 0) {
        *error = ERROR_INVALID_INPUT;
        setErrorMessage("Invalid points array");
        return NULL;
    }
    
    if (count > MAX_ARRAY_SIZE) {
        *error = ERROR_ARRAY_BOUNDS;
        setErrorMessage("Too many points");
        return NULL;
    }
    
    struct Point* centroid = (struct Point*)malloc(sizeof(struct Point));
    if (!centroid) {
        *error = ERROR_MEMORY_ALLOCATION;
        setErrorMessage("Failed to allocate memory for centroid");
        return NULL;
    }
    
    double totalWeight = 0;
    double weightedX = 0;
    double weightedY = 0;
    
    for (int i = 0; i < count; i++) {
        if (points[i].weight <= 0) {
            *error = ERROR_INVALID_INPUT;
            setErrorMessage("Invalid point weight");
            free(centroid);
            return NULL;
        }
        
        totalWeight += points[i].weight;
        weightedX += points[i].x * points[i].weight;
        weightedY += points[i].y * points[i].weight;
        
        // 检查累积计算是否溢出
        if (isnan(totalWeight) || isnan(weightedX) || isnan(weightedY) ||
            isinf(totalWeight) || isinf(weightedX) || isinf(weightedY)) {
            *error = ERROR_OVERFLOW;
            setErrorMessage("Centroid calculation overflow");
            free(centroid);
            return NULL;
        }
    }
    
    if (totalWeight == 0) {
        *error = ERROR_INVALID_INPUT;
        setErrorMessage("Total weight is zero");
        free(centroid);
        return NULL;
    }
    
    centroid->x = (int)(weightedX / totalWeight);
    centroid->y = (int)(weightedY / totalWeight);
    centroid->weight = totalWeight / count;
    strcpy(centroid->label, "Centroid");
    
    return centroid;
}